/*
 * JBoss, Home of Professional Open Source
 * Copyright 2012, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the 
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,  
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.as.quickstarts.ejb.remote.client;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Set;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.as.quickstarts.ejb.remote.stateless.RemoteSalesService;

import com.marshall.CollectionReference;
import com.marshall.Sale;
import com.marshall.SaleService;
import com.marshall.WeightService;
import com.marshall.WeightServiceValue;

/**
 * A sample program which acts a remote client for a EJB deployed on AS7 server.
 * This program shows how to lookup stateful and stateless beans via JNDI and
 * then invoke on them
 * 
 * @author Jaikiran Pai, mzottner@redhat.com
 */
public class RemoteEJBClient {

	public static void main(String[] args) throws Exception {
		// Invoke a stateless bean
		invokeStatelessBean();
	}

	/**
	 * Looks up a stateless bean and invokes on it
	 * 
	 * @throws NamingException
	 */
	private static void invokeStatelessBean() throws NamingException {
		// Let's lookup the remote stateless calculator
		final RemoteSalesService statelessEJB = lookupRemoteStatelessSalesService();

		// statelessRemoteCalculator.serializeCrash("");//new
		// ClientSidedObject());

		Thread t1 = new Thread(new Runnable() {

			@Override
			public void run() {
				System.out.println(">>> Thread T1 - before invocation of \"serializeCrash\"");
				// Object result = statelessRemoteCalculator.waitAWhile();

				Sale sale = new Sale();
				Set<SaleService> saleServices = new HashSet<SaleService>();
				sale.setSaleServices(saleServices);

				WeightService ws1 = new WeightService();
				WeightService ws2 = new WeightService();

				saleServices.add(ws1);
				saleServices.add(ws2);

				CollectionReference swv1 = new CollectionReference();
				CollectionReference swv2 = new CollectionReference();
				
				swv1.initSnapshot(new WeightServiceValue(ws1));
				swv2.initSnapshot(new WeightServiceValue(ws2));

				ws1.setWeightServiceValues(swv1);
				ws2.setWeightServiceValues(swv2);

				// Does the serialized object will be remote serialized.
				statelessEJB.remoteSerialization(sale);
				System.out.println("<<< Thread T1 - after invocation of \"serializeCrash\"");
			}
		});

		t1.start();
	}

	/**
	 * Looks up and returns the proxy to remote stateless calculator bean
	 * 
	 * @return
	 * @throws NamingException
	 */
	private static RemoteSalesService lookupRemoteStatelessSalesService() throws NamingException {
		final Hashtable<String, String> jndiProperties = new Hashtable<String, String>();
		jndiProperties.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
		final Context context = new InitialContext(jndiProperties);

		// let's do the lookup
		return (RemoteSalesService) context.lookup("ejb:/jboss-as-ejb-remote-app/RemoteSalesBean!" + RemoteSalesService.class.getName());
	}

}
